@@ -153,11 +153,13 @@ gql_get_project_info = """
 
 gql_get_project_technologies = """
 {{
-   technologiesByCategory(projectId: "{0}"){{
+   project(id: "{0}"){{
+   technologiesByCategory {{
       jobCategory,
       technologies{{
         id
         }}
+      }}
    }}
 }} 
 """
@@ -346,72 +348,101 @@ gql_stop_job_instance = """
   """
 
 gql_edit_job = """
-  mutation{{
-    editJob(job: {{
-      id: "{0}"
-      {1}
-    }}
-    ) {{
-        id,
-        name,
-        description,
-        isScheduled,
-        cronScheduling,
+mutation editJobMutation($id: UUID!, $name: String, $description: String, 
+                         $isScheduled: Boolean!, $cronScheduling: Cron, $scheduleTimezone: TimeZone,
+                         $alerting: JobPipelineAlertingInput, $resources: JobResourceInput) {
+    editJob(job: {
+        id: $id
+        name: $name
+        description: $description
+        isScheduled: $isScheduled
+        cronScheduling: $cronScheduling
+        scheduleTimezone: $scheduleTimezone
+        alerting: $alerting
+        resources: $resources
+    }){
+        id
+        name
+        description
+        isScheduled
+        cronScheduling
         scheduleTimezone
-        resources{{
-        cpu {{
-            request,
-            limit}},
-        memory{{
-            request,
-            limit}}
-        }}
-      }}
-  }}
+        resources{
+            cpu {
+                request
+                limit}
+            memory{
+                request
+                limit}
+        }
+        alerting{
+            emails
+            statusList
+        }
+    }
+}
 """
 
-gql_create_job = """{{"operationName": "createJobMutation",\
-                   "variables": {{\
-                       "job": {{\
-                           "projectId": "{1}",\
-                           "name": "{0}",\
-                           "description": "{2}",\
-                           "category": "{3}",\
-                           "technology": {{"id":"{4}"}},\
-                           "isStreaming": false,\
-                           {9}, \
-                           "resources": {10}, \
-                           "doesUseGPU": false\
-                       }},\
-                       "jobVersion": {{\
-                           "runtimeVersion": "{5}",\
-                           "commandLine": "{6}",\
-                           {8}\
-                           "dockerInfo": null,\
-                           "releaseNote": "{7}"\
-                       }},\
-                       "file":null\
-                   }},\
-                   "query": "mutation createJobMutation($job: JobInput!, $jobVersion: JobVersionInput!, $file: Upload) \
-                   {{\\n  createJob(job: $job, jobVersion: $jobVersion, file: $file) \
-                   {{\\n    id\\n    versions {{\\n      number\\n      __typename\\n    }}\\n   \
-                    __typename\\n  }}\\n}}\\n"}}"""
+gql_create_job = """
+mutation createJobMutation($projectId: UUID!, $name: String!, $description: String, $category: String!,
+                           $isScheduled: Boolean!, $cronScheduling: Cron, $scheduleTimezone: TimeZone
+                           $technologyId: UUID!, 
+                           $alerting: JobPipelineAlertingInput, $resources: JobResourceInput,
+                           $releaseNote: String, $runtimeVersion: String, $commandLine: String,
+                           $dockerInfo: JobDockerInput, $file: Upload) {{
+    createJob(job: {{
+            projectId: $projectId
+            name: $name
+            description: $description
+            category: $category
+            technology: {{
+                id: $technologyId
+            }}
+            isStreaming: false
+            isScheduled: $isScheduled
+            cronScheduling: $cronScheduling
+            scheduleTimezone: $scheduleTimezone
+            alerting: $alerting
+            resources: $resources
+            doesUseGPU: false
+        }} 
+        jobVersion: {{
+            releaseNote: $releaseNote
+            runtimeVersion: $runtimeVersion
+            commandLine: $commandLine
+            {extra_technology}
+            dockerInfo: $dockerInfo
+        }}
+        file: $file){{
+        id
+        versions {{
+            number
+            __typename
+        }}
+        __typename
+    }}
+}}
+"""
 
-gql_upgrade_job = """{{"operationName": "addJobVersionMutation",\
-    "variables": {{\
-        "jobId": "{0}",\
-        "jobVersion": {{\
-            "runtimeVersion": "{1}",\
-            "commandLine": "{2}",\
-            {4}\
-            "dockerInfo": null,\
-            "releaseNote": "{3}"\
-        }},\
-        "file": null\
-    }},\
-    "query": "mutation addJobVersionMutation($jobId: UUID!, $jobVersion: JobVersionInput!, $file: Upload) \
-    {{\\n  addJobVersion(jobId: $jobId, jobVersion: $jobVersion, file: $file) \
-    {{\\n    number\\n    __typename\\n  }}\\n}}\\n"}}"""
+gql_upgrade_job = """
+mutation addJobVersionMutation($jobId: UUID!, $releaseNote: String, $runtimeVersion: String, $commandLine: String,
+                               $usePreviousArtifact: Boolean, $dockerInfo: JobDockerInput, $file: Upload) {{
+    addJobVersion(
+        jobId: $jobId
+        jobVersion: {{
+            releaseNote: $releaseNote
+            runtimeVersion: $runtimeVersion
+            commandLine: $commandLine
+            {extra_technology}
+            dockerInfo: $dockerInfo
+            usePreviousArtifact: $usePreviousArtifact
+        }}
+        file: $file){{
+            number
+            __typename
+    }}
+}}
+"""
 
 gql_get_info_job = """query {{
   job(id:"{0}"){{
@@ -453,10 +484,11 @@ gql_get_info_job = """query {{
 }}
 """
 
-gql_extra_technology = '"extraTechnology": {{\
-                           "language": "{0}",\
-                           "version": "{1}"\
-                       }},'
+gql_extra_technology = """
+    extraTechnology: {{
+        language: "{0}"
+        version: "{1}"
+    }}"""
 
 gql_delete_job = """
   mutation {{
@@ -649,30 +681,27 @@ gql_stop_pipeline_instance = """
   """
 
 gql_edit_pipeline = """
-  mutation($id: UUID!, $name: String, $description: String, $emails: [Email!], $statusList: [InstanceStatus!]!,
+  mutation($id: UUID!, $name: String, $description: String, $alerting: JobPipelineAlertingInput,
           $isScheduled: Boolean, $cronScheduling: Cron, $scheduleTimezone:TimeZone)  {
     editPipeline(pipeline: {
-        id: $id,
-        name: $name,
-        description: $description,
-        alerting: {
-          emails: $emails,
-          statusList: $statusList
-        }
-        isScheduled: $isScheduled,
-        cronScheduling: $cronScheduling,
+        id: $id
+        name: $name
+        description: $description
+        alerting:  $alerting
+        isScheduled: $isScheduled
+        cronScheduling: $cronScheduling
         scheduleTimezone: $scheduleTimezone
       })
     {
-      id,
-      name,
-      description,
+      id
+      name
+      description
       alerting{
-        emails,
+        emails
         statusList
       }
-      isScheduled,
-      cronScheduling,
+      isScheduled
+      cronScheduling
       scheduleTimezone
     }
   }
