@@ -3,7 +3,6 @@ Saagie API object to interact with Saagie API in Python (API for
 Projects & Jobs - to interact with the manager API, see the manager subpackage)
 
 """
-import json
 import logging
 import time
 import re
@@ -18,6 +17,7 @@ from gql.transport.requests import RequestsHTTPTransport
 from .auth import *
 from .gql_template import *
 from .graph_pipeline import *
+import deprecation
 
 
 class SaagieApi:
@@ -26,7 +26,7 @@ class SaagieApi:
     subpackage)
     """
 
-    def __init__(self, url_saagie, id_platform, user, password, realm):
+    def __init__(self, url_saagie, id_platform, user, password, realm, retries=0):
         """
         Parameters
         ----------
@@ -35,11 +35,13 @@ class SaagieApi:
         id_platform : int or str
             Platform Id  (see README on how to find it)
         user : str
-            username to login with
+            username to log in with
         password : str
-            password to login with
+            password to log in with
         realm : str
             Saagie realm  (see README on how to find it)
+        retries : int
+            Pre-setup of the requests’ Session for performing retries
         """
         if not url_saagie.endswith('/'):
             url_saagie += '/'
@@ -49,16 +51,18 @@ class SaagieApi:
         self.realm = realm
         self.login = user
         self.password = password
+        self.retries = retries
         self.auth = BearerAuth(self.realm, self.url_saagie,
                                self.id_platform, self.login, self.password)
         url = self.url_saagie + self.suffix_api + 'platform/'
         url += str(self.id_platform) + '/graphql'
         self._url = url
         self._transport = RequestsHTTPTransport(
-            url=url,
+            url=self._url,
             auth=self.auth,
             use_json=True,
-            verify=False
+            verify=False,
+            retries=self.retries
         )
         self.client = Client(
             transport=self._transport,
@@ -78,6 +82,10 @@ class SaagieApi:
             fetch_schema_from_transport=True
         )
 
+        # Valid status list of alerting
+        self.valid_status_list = ["REQUESTED", "QUEUED", "RUNNING", "FAILED", "KILLED",
+                                  "KILLING", "SUCCEEDED", "UNKNOWN", "AWAITING", "SKIPPED"]
+
     @classmethod
     def easy_connect(cls, url_saagie_platform, user, password):
         """
@@ -91,12 +99,12 @@ class SaagieApi:
         url_saagie_platform : str
             Complete platform URL (eg: https://saagie-workspace.prod.saagie.io/projects/platform/6/)
         user : str
-            username to login with
+            username to log in with
         password : str
-            password to login with
+            password to log in with
         """
         url_regex = re.compile(
-            r"(https:\/\/(\w+)-(?:\w|\.)+)\/projects\/platform\/(\d+)")
+            r"(https://(\w+)-(?:\w|\.)+)/projects/platform/(\d+)")
         m = url_regex.match(url_saagie_platform)
         if bool(m):
             url_saagie = m.group(1)
@@ -312,6 +320,25 @@ class SaagieApi:
     # ######################################################
     # ###                    projects                   ####
     # ######################################################
+    def get_project_id(self, project_name):
+        """Get the project id with the project name
+        Parameters
+        ----------
+        project_name : str
+            Name of your project
+        Returns
+        -------
+        str
+            Project UUID
+        """
+        projects = self.get_projects_info()["projects"]
+        project = list(filter(lambda p: p["name"] == project_name, projects))
+        if project:
+            project_id = project[0]["id"]
+            return project_id
+
+        else:
+            raise NameError(f"Project {project_name} does not exist or you don't have permission to see it.")
 
     def get_projects_info(self):
         """Get information for all projects (id, name, creator, description,
@@ -359,7 +386,7 @@ class SaagieApi:
             Dict of available technologies
         """
         query = gql(gql_get_project_technologies.format(project_id))
-        return self.client.execute(query)
+        return self.client.execute(query)['project']
 
     def create_project(self, name, group=None, role="Manager", description=""):
         """Create a new project on the platform
@@ -578,8 +605,9 @@ class SaagieApi:
         query = gql(gql_stop_job_instance.format(job_instance_id))
         return self.client.execute(query)
 
-    def edit_job(self, job_id, job_name=None, description=None, is_scheduled=False,
-                 cron_scheduling=None, schedule_timezone="UTC", resources=None):
+    def edit_job(self, job_id, job_name=None, description=None, is_scheduled=None,
+                 cron_scheduling=None, schedule_timezone="UTC", resources=None,
+                 emails=None, status_list=["FAILED"]):
         """Edit a job
 
         Parameters
@@ -587,54 +615,108 @@ class SaagieApi:
         job_id : str
             UUID of your job (see README on how to find it)
         job_name : str, optional
-            Seconds to wait between two state checks
+            Job name
+            If not filled, defaults to current value, else it will change the job's name
         description : str, optional
             Description of job
+            if not filled, defaults to current value, else it will change the description of the pipeline
         is_scheduled : bool, optional
             True if the job is scheduled, else False
+            if not filled, defaults to current value
         cron_scheduling : str, optional
-            Scheduling cron format
+            Scheduling CRON format
+            When is_scheduled is set to True, it will be mandatory to fill this value
+            if not filled, defaults to current value
+            Example: "0 0 * * *" (for every day At 00:00)
         schedule_timezone : str, optional
             Timezone of the scheduling
+            Example: "UTC", "Pacific/Pago_Pago"
         resources : dict, optional
             CPU, memory limit and requests
+            if not filled, defaults to current value
             Example: {"cpu":{"request":0.5, "limit":2.6},"memory":{"request":1.0}}
+        emails: List[String], optional
+            Emails to receive alerts for the job, each item should be a valid email,
+            If you want to remove alerting, please set emails to [] or list()
+            if not filled, defaults to current value
+        status_list: List[String], optional
+            Receive an email when the job status change to a specific status
+            Each item of the list should be one of these following values: "REQUESTED", "QUEUED",
+            "RUNNING", "FAILED", "KILLED", "KILLING", "SUCCEEDED", "UNKNOWN", "AWAITING", "SKIPPED"
 
         Returns
         -------
         dict
             Dict of job information
         """
-        gql_payload = []
+        params = {"id": job_id}
+        previous_job_version = self.get_job_info(job_id)["job"]
+
         if job_name:
-            gql_payload.append(f'name: "{job_name}"')
+            params["name"] = job_name
+        else:
+            params["name"] = previous_job_version["name"]
 
         if description:
-            gql_payload.append(f'description: "{description}"')
+            params["description"] = description
+        else:
+            params["description"] = previous_job_version["description"]
+
+        if resources:
+            params["resources"] = resources
+        else:
+            params["resources"] = previous_job_version["resources"]
 
         if is_scheduled:
-            gql_payload.append(f'isScheduled: true')
+            params["isScheduled"] = True
 
             if cron_scheduling and croniter.is_valid(cron_scheduling):
-                gql_payload.append(f'cronScheduling: "{cron_scheduling}"')
+                params["cronScheduling"] = cron_scheduling
             else:
                 raise RuntimeError(f"{cron_scheduling} is not valid cron format")
 
             if schedule_timezone in list(pytz.all_timezones):
-                gql_payload.append(f'scheduleTimezone: "{schedule_timezone}"')
+                params["scheduleTimezone"] = schedule_timezone
             else:
                 raise RuntimeError("Please specify a correct timezone")
 
+        elif is_scheduled == False:
+            params["isScheduled"] = False
+
         else:
-            gql_payload.append(f'isScheduled: false')
+            params["isScheduled"] = previous_job_version["isScheduled"]
+            params["cronScheduling"] = previous_job_version["cronScheduling"]
+            params["scheduleTimezone"] = previous_job_version["scheduleTimezone"]
 
-        if resources:
-            resources_str = json.dumps(resources).replace("\"", "")
-            gql_payload.append(f'resources: {resources_str}')
+        if emails:
+            wrong_status_list = []
 
-        gql_payload_str = ", ".join(gql_payload)
-        query = gql(gql_edit_job.format(job_id, gql_payload_str))
-        return self.client.execute(query)
+            for item in status_list:
+                if item not in self.valid_status_list:
+                    wrong_status_list.append(item)
+            if wrong_status_list:
+                raise RuntimeError(f"The following status are not valid: {wrong_status_list}. "
+                                   f"Please make sure that each item of the parameter status_list should be "
+                                   f"one of the following values: 'REQUESTED', 'QUEUED', 'RUNNING', "
+                                   f"'FAILED', 'KILLED', 'KILLING', 'SUCCEEDED', 'UNKNOWN', 'AWAITING', 'SKIPPED'")
+
+            else:
+                params["alerting"] = {
+                    "emails": emails,
+                    "statusList": status_list
+                }
+        elif type(emails) == list:
+            params["alerting"] = None
+        else:
+            previous_alerting = previous_job_version["alerting"]
+            if previous_alerting:
+                params["alerting"] = {
+                    "emails": previous_alerting["emails"],
+                    "statusList": previous_alerting["statusList"]
+                }
+
+        query = gql(gql_edit_job)
+        return self.client.execute(query, variable_values=params)
 
     def create_job(self, job_name, project_id, file=None, description='',
                    category='Processing', technology='python',
@@ -642,21 +724,13 @@ class SaagieApi:
                    runtime_version='3.6',
                    command_line='python {file} arg1 arg2', release_note='',
                    extra_technology='', extra_technology_version='',
-                   cron_scheduling=None, schedule_timezone="UTC", resources=None):
+                   cron_scheduling=None, schedule_timezone="UTC", resources=None,
+                   emails=None, status_list=["FAILED"]):
         """Create job in given project
 
         NOTE
         ----
-        - Since gql does not support multipart graphQL requests, requests
-          package is used for now. The create job graphQL API takes a multipart
-          graphQL request to upload a file.
-          See https://github.com/jaydenseric/graphql-multipart-request-spec
-          for more details
-        - 2020-06-08 : multipart graphQL support will probably be implemented
-          in gql in the near future.
-          See https://github.com/graphql-python/gql/issues/68 to follow this
-          work.
-        - Tested with python and spark jobs
+        - Only work for gql>=3.0.0
 
         Parameters
         ----------
@@ -689,22 +763,29 @@ class SaagieApi:
             needed
         cron_scheduling : str, optional
             Scheduling CRON format
+            Example: "0 0 * * *" (for every day At 00:00)
         schedule_timezone : str, optional
             Timezone of the scheduling
+            Example: "UTC", "Pacific/Pago_Pago"
         resources : dict, optional
             CPU, memory limit and requests
             Example: {"cpu":{"request":0.5, "limit":2.6},"memory":{"request":1.0}}
+        emails: List[String], optional
+            Emails to receive alerts for the job, each item should be a valid email
+        status_list: List[String], optional
+            Receive an email when the job status change to a specific status
+            Each item of the list should be one of these following values: "REQUESTED", "QUEUED",
+            "RUNNING", "FAILED", "KILLED", "KILLING", "SUCCEEDED", "UNKNOWN", "AWAITING", "SKIPPED"
 
         Returns
         -------
         dict
             Dict of job information
 
-        Raises
-        ------
-        requests.exceptions.RequestException
-            When requests fails
         """
+        params = {
+            "projectId": project_id, "name": job_name, "description": description, "category": category,
+            "releaseNote": release_note, "runtimeVersion": runtime_version, "commandLine": command_line}
 
         technologies_for_project = self.get_project_technologies(project_id)['technologiesByCategory']
         technologies_for_project_and_category = [
@@ -737,6 +818,7 @@ class SaagieApi:
                 f"and for the {technology_catalog} catalog")
         else:
             technology_id = technology_in_catalog[0]
+            params["technologyId"] = technology_id
 
         if extra_technology != '':
             extra_tech = gql_extra_technology.format(extra_technology,
@@ -744,41 +826,45 @@ class SaagieApi:
         else:
             extra_tech = ''
 
-        if resources is None:
-            resources = {}
-        gql_scheduling_payload = []
+        if emails:
+            wrong_status_list = []
+
+            for item in status_list:
+                if item not in self.valid_status_list:
+                    wrong_status_list.append(item)
+            if wrong_status_list:
+                raise RuntimeError(f"The following status are not valid: {wrong_status_list}. "
+                                   f"Please make sure that each item of the parameter status_list should be "
+                                   f"one of the following values: 'REQUESTED', 'QUEUED', 'RUNNING', "
+                                   f"'FAILED', 'KILLED', 'KILLING', 'SUCCEEDED', 'UNKNOWN', 'AWAITING', 'SKIPPED'")
+
+            else:
+                params["alerting"] = {
+                    "emails": emails,
+                    "statusList": status_list
+                }
 
         if cron_scheduling:
-            gql_scheduling_payload.append(f'"isScheduled": true')
+            params["isScheduled"] = True
 
             if croniter.is_valid(cron_scheduling):
-                gql_scheduling_payload.append(f'"cronScheduling": "{cron_scheduling}"')
+                params["cronScheduling"] = cron_scheduling
             else:
                 raise RuntimeError(f"{cron_scheduling} is not valid cron format")
 
             if schedule_timezone in list(pytz.all_timezones):
-                gql_scheduling_payload.append(f'"scheduleTimezone": "{schedule_timezone}"')
+                params["scheduleTimezone"] = schedule_timezone
             else:
                 raise RuntimeError("Please specify a correct timezone")
 
         else:
-            gql_scheduling_payload.append(f'"isScheduled": false')
+            params["isScheduled"] = False
 
-        gql_scheduling_payload_str = ", ".join(gql_scheduling_payload)
-        resources_str = json.dumps(resources)
-        payload_str = gql_create_job.format(job_name,
-                                            project_id,
-                                            description,
-                                            category,
-                                            technology_id,
-                                            runtime_version,
-                                            command_line,
-                                            release_note,
-                                            extra_tech,
-                                            gql_scheduling_payload_str,
-                                            resources_str)
+        if resources:
+            params["resources"] = resources
 
-        return self.__launch_request(file, payload_str)
+        payload_str = gql_create_job.format(extra_technology=extra_tech)
+        return self.__launch_request(file, payload_str, params)
 
     def get_job_info(self, job_id):
         """Get job's info
@@ -796,10 +882,11 @@ class SaagieApi:
         """
         query = gql_get_info_job.format(job_id)
         return self.client.execute(gql(query))
-        
+
     def upgrade_job(self, job_id, file=None, use_previous_artifact=False, runtime_version='3.6',
-                   command_line='python {file} arg1 arg2', release_note=None):
-        """Ugrade a job
+                    command_line='python {file} arg1 arg2', release_note=None,
+                    extra_technology='', extra_technology_version=''):
+        """Upgrade a job
 
         Parameters
         ----------
@@ -815,11 +902,18 @@ class SaagieApi:
             Command line
         release_note: str (optional)
             Release note
+        extra_technology: str (optional)
+            Extra technology when needed (spark jobs). If not needed, leave to
+            empty string or the request will not work
+        extra_technology_version: str (optional)
+            Version of the extra technology. Leave to empty string when not
+            needed
 
         Returns
         -------
         dict
             Dict with version number
+            Example: {'addJobVersion': {'number': 5, '__typename': 'JobVersion'}}
 
         """
 
@@ -827,41 +921,62 @@ class SaagieApi:
         technology_id = self.get_job_info(job_id)["job"]["technology"]["id"]
         available_runtimes = [c["label"] for c in self.get_runtimes(technology_id)["technology"]["contexts"]]
         if runtime_version not in available_runtimes:
-            raise RuntimeError(f"Specified runtime does not exist ({runtime_version}). Available runtimes : {','.join(available_runtimes)}.")
+            raise RuntimeError(
+                f"Specified runtime does not exist ({runtime_version}). "
+                f"Available runtimes : {','.join(available_runtimes)}.")
 
         if file and use_previous_artifact:
-            logging.warn("You can not specify a file and use the previous artifact. By default, the specified file will be used.")
-        use_previous_artifact_str = f'"usePreviousArtifact": {"null" if use_previous_artifact else "false"},'
+            logging.warning(
+                "You can not specify a file and use the previous artifact. "
+                "By default, the specified file will be used.")
 
-        payload_str = gql_upgrade_job.format(job_id, runtime_version, command_line, release_note, use_previous_artifact_str)
+        if extra_technology != '':
+            extra_tech = gql_extra_technology.format(extra_technology,
+                                                     extra_technology_version)
+        else:
+            extra_tech = ''
+        payload_str = gql_upgrade_job.format(extra_technology=extra_tech)
+        params = {"jobId": job_id, "releaseNote": release_note, "runtimeVersion": runtime_version,
+                  "commandLine": command_line, "usePreviousArtifact": use_previous_artifact}
 
-        return self.__launch_request(file, payload_str)
+        return self.__launch_request(file, payload_str, params)
 
-    def __launch_request(self, file, payload_str):
+    def __launch_request(self, file, payload_str, params):
+        """Launch a GQL request with specified file, payload and params
+        GQL3 needed to use this function
+        Parameters
+        ----------
+        file : str
+            Path to your file
+        payload_str : str
+            Payload to send
+        params: dict
+            variable values to pass to the GQL request
+        Returns
+        -------
+        dict
+            Dict of the request response
+        """
         if file:
             file = Path(file)
             with file.open(mode='rb') as f:
-                files = {
-                    '1': (file.name, f),
-                    'operations': (None, payload_str),
-                    'map': (None, '{ "1": ["variables.file"] }'),
-                }
-                response = requests.post(self._url,
-                                         files=files,
-                                         auth=self.auth,
-                                         verify=False)
+                params["file"] = f
+                try:
+                    req = self.client.execute(gql(payload_str), variable_values=params, upload_files=True)
+                    res = {"data": req}
+                except Exception as e:
+                    logging.error(f"Something went wrong: {e}")
+                    raise e
+                return res
+
         else:
-            payload = json.loads(payload_str)
-            response = requests.post(self._url,
-                                     json=payload,
-                                     auth=self.auth,
-                                     verify=False)
-        
-        if response:
-            return json.loads(response.content)
-        else:
-            m = f"Requests failed with status_code :'{response.status_code}'"
-            raise requests.exceptions.RequestException(m)
+            try:
+                req = self.client.execute(gql(payload_str), variable_values=params)
+                res = {"data": req}
+            except Exception as e:
+                logging.error(f"Something went wrong: {e}")
+                raise e
+            return res
 
     def get_job_id(self, job_name, project_name):
         """Get the job id with the job name and project name
@@ -875,26 +990,23 @@ class SaagieApi:
 
         Returns
         -------
-        dict
+        str
             Job UUID
 
         """
-        projects = self.get_projects_info()["projects"]
-        project = list(filter(lambda p: p["name"] == project_name, projects))
-        if project:
-            project_id = project[0]["id"]
-            jobs = self.get_project_jobs(project_id, instances_limit=1)["jobs"]
-            job = list(filter(lambda j: j["name"] == job_name, jobs))
-            if job:
-                return job[0]["id"]
-            else:
-                raise NameError(f"Job {job_name} does not exist.")
+        project_id = self.get_project_id(project_name)
+        jobs = self.get_project_jobs(project_id, instances_limit=1)["jobs"]
+        job = list(filter(lambda j: j["name"] == job_name, jobs))
+        if job:
+            return job[0]["id"]
         else:
-            raise NameError(f"Project {project_name} does not exist or you don't have permission to see it.")
-    
+            raise NameError(f"Job {job_name} does not exist.")
+
     def upgrade_job_by_name(self, job_name, project_name, file=None, use_previous_artifact=False, runtime_version='3.6',
-                   command_line='python {file} arg1 arg2', release_note=None):
-        """Ugrade a job
+                            command_line='python {file} arg1 arg2', release_note=None,
+                            extra_technology='', extra_technology_version=''
+                            ):
+        """Upgrade a job
 
         Parameters
         ----------
@@ -912,15 +1024,23 @@ class SaagieApi:
             Command line
         release_note: str (optional)
             Release note
+        extra_technology: str (optional)
+            Extra technology when needed (spark jobs). If not needed, leave to
+            empty string or the request will not work
+        extra_technology_version: str (optional)
+            Version of the extra technology. Leave to empty string when not
+            needed
 
         Returns
         -------
         dict
             Dict with version number
+            Example: {'addJobVersion': {'number': 5, '__typename': 'JobVersion'}}
 
         """
         job_id = self.get_job_id(job_name, project_name)
-        return self.upgrade_job(job_id, file, use_previous_artifact, runtime_version, command_line, release_note)
+        return self.upgrade_job(job_id, file, use_previous_artifact, runtime_version, command_line, release_note,
+                                extra_technology, extra_technology_version)
 
     def delete_job(self, job_id):
         """Delete a given job
@@ -963,7 +1083,9 @@ class SaagieApi:
         dict
             Dict of webApp information
         """
-        regex_error_missing_technology = r"io\.saagie\.projectsandjobs\.domain\.exception\.NonExistingTechnologyException: Technology \S{8}-\S{4}-\S{4}-\S{4}-\S{12} does not exist"
+        regex_error_missing_technology = r"io\.saagie\.projectsandjobs\.domain\.exception" \
+                                         r"\.NonExistingTechnologyException: Technology \S{8}-\S{4}-\S{4}-\S{4}-\S{" \
+                                         r"12} does not exist "
         instances_limit_request = f" (limit: {str(instances_limit)})" if instances_limit != -1 else ""
 
         query = gql(gql_get_project_web_apps.format(project_id,
@@ -1069,39 +1191,107 @@ class SaagieApi:
         query = gql(gql_stop_pipeline_instance.format(pipeline_instance_id))
         return self.client.execute(query)
 
-    # TO DETAIL!
-    # Detail pipeline dict parameters, or explode the number of parameters to
-    # have a more comprehensible method
-    def edit_pipeline(self, pipeline_id, name=None, description=None, emails=None, status_list=None, is_scheduled=None,
-             cron_scheduling=None, schedule_timezone=None):
-        """Edit a given pipeline
+    def edit_pipeline(self, pipeline_id, name=None, description=None, emails=None,
+                      status_list=["FAILED"], is_scheduled=None,
+                      cron_scheduling=None, schedule_timezone="UTC"):
+        """Edit a pipeline
         NB : You can only edit pipeline if you have at least the editor role on
         the project
+        Parameters
+        ----------
+        pipeline_id : str
+            UUID of your pipeline (see README on how to find it)
+        name : str, optional
+            Pipeline name,
+            if not filled, defaults to current value, else it will change the pipeline name
+        description : str, optional
+            Description of the pipeline
+            if not filled, defaults to current value, else it will change the description of the pipeline
+        is_scheduled : bool, optional
+            True if the job is scheduled, else False
+            if not filled, defaults to current value
+        cron_scheduling : str, optional
+            Scheduling CRON format
+            When is_scheduled is set to True, it will be mandatory to fill this value
+            if not filled, defaults to current value
+            Example: "0 0 * * *" (for every day At 00:00)
+        schedule_timezone : str, optional
+            Timezone of the scheduling
+            Example: "UTC", "Pacific/Pago_Pago"
+        emails: List[String], optional
+            Emails to receive alerts for the job, each item should be a valid email,
+            If you want to remove alerting, please set emails to [] or list()
+            if not filled, defaults to current value
+        status_list: List[String], optional
+            Receive an email when the job status change to a specific status
+            Each item of the list should be one of these following values: "REQUESTED", "QUEUED",
+            "RUNNING", "FAILED", "KILLED", "KILLING", "SUCCEEDED", "UNKNOWN", "AWAITING", "SKIPPED"
 
-        Args:
-            pipeline_id (str): mandatory pipeline id to edit
-            name (str): if not filled, defaults to current value
-            description (str): if not filled, defaults to current value
-            emails (list, optional): if not filled, defaults to current value
-            status_list (list, optional): if not filled, defaults to current value
-            is_scheduled (bool, optional): if not filled, defaults to current value
-            cron_scheduling (str, optional): if not filled, defaults to current value
-            schedule_timezone (str, optional): if not filled, defaults to current value
-
-        Returns:
-            [type]: [description]
+        Returns
+        -------
+        dict
+            Dict of pipeline information
         """
-        pipeline_info = self.get_project_pipeline(pipeline_id)
-        if not name: name = pipeline_info["graphPipeline"]["name"]
-        if not description: description = pipeline_info["graphPipeline"]["description"]
-        if not emails: emails = pipeline_info["graphPipeline"]["alerting"]["emails"]
-        if not status_list: status_list = pipeline_info["graphPipeline"]["alerting"]["statusList"]
-        if not is_scheduled: is_scheduled = pipeline_info["graphPipeline"]["isScheduled"]
-        if not cron_scheduling: cron_scheduling = pipeline_info["graphPipeline"]["cronScheduling"]
-        if not schedule_timezone: schedule_timezone = pipeline_info["graphPipeline"]["scheduleTimezone"]
+        params = {"id": pipeline_id}
+        previous_pipeline_info = self.get_project_pipeline(pipeline_id)["graphPipeline"]
 
-        params = {'id': pipeline_id, 'name': name, 'description': description, 'emails': emails, 'statusList': status_list,
-                'isScheduled': is_scheduled, 'cronScheduling': cron_scheduling, 'scheduleTimezone': schedule_timezone}
+        if name:
+            params["name"] = name
+        else:
+            params["name"] = previous_pipeline_info["name"]
+
+        if description:
+            params["description"] = description
+        else:
+            params["description"] = previous_pipeline_info["description"]
+
+        if is_scheduled:
+            params["isScheduled"] = True
+
+            if cron_scheduling and croniter.is_valid(cron_scheduling):
+                params["cronScheduling"] = cron_scheduling
+            else:
+                raise RuntimeError(f"{cron_scheduling} is not valid cron format")
+
+            if schedule_timezone in list(pytz.all_timezones):
+                params["scheduleTimezone"] = schedule_timezone
+            else:
+                raise RuntimeError("Please specify a correct timezone")
+
+        elif is_scheduled == False:
+            params["isScheduled"] = False
+
+        else:
+            params["isScheduled"] = previous_pipeline_info["isScheduled"]
+            params["cronScheduling"] = previous_pipeline_info["cronScheduling"]
+            params["scheduleTimezone"] = previous_pipeline_info["scheduleTimezone"]
+
+        if emails:
+            wrong_status_list = []
+
+            for item in status_list:
+                if item not in self.valid_status_list:
+                    wrong_status_list.append(item)
+            if wrong_status_list:
+                raise RuntimeError(f"The following status are not valid: {wrong_status_list}. "
+                                   f"Please make sure that each item of the parameter status_list should be "
+                                   f"one of the following values: 'REQUESTED', 'QUEUED', 'RUNNING', "
+                                   f"'FAILED', 'KILLED', 'KILLING', 'SUCCEEDED', 'UNKNOWN', 'AWAITING', 'SKIPPED'")
+
+            else:
+                params["alerting"] = {
+                    "emails": emails,
+                    "statusList": status_list
+                }
+        elif type(emails) == list:
+            params["alerting"] = None
+        else:
+            previous_alerting = previous_pipeline_info["alerting"]
+            if previous_alerting:
+                params["alerting"] = {
+                    "emails": previous_pipeline_info["emails"],
+                    "statusList": previous_pipeline_info["statusList"]
+                }
 
         query = gql(gql_edit_pipeline)
         return self.client.execute(query, variable_values=params)
@@ -1168,6 +1358,9 @@ class SaagieApi:
             print('Current state : ' + state)
         return state
 
+    @deprecation.deprecated(deprecated_in="Saagie 2.2.1",
+                            details="This deprecated endpoint allows to create only linear pipeline. "
+                                    "To create graph pipelines, use the endpoint `createGraphPipeline` instead.")
     def create_pipeline(self, name, project_id, jobs_id, description=""):
         """
         Create a pipeline in a given project
@@ -1226,7 +1419,13 @@ class SaagieApi:
         project_id : str
             UUID of your project (see README on how to find it)
         graph_pipeline : GraphPipeline
-
+            Example: If you want to create a simple pipeline with 2 jobs that started by job_node_1,
+            you can use the following example
+                job_node1 = JobNode(job_id_1)
+                job_node2 = JobNode(job_id_2)
+                job_node1.add_next_node(job_node2) # Indicates that the job_node_1 is followed by job_node_2
+                graph_pipeline = GraphPipeline()
+                graph_pipeline.add_root_node(job_node1) # Indicates the pipeline will started with job_node1
         description : str, optional
             Description of the pipeline
         release_note: str, optional
@@ -1245,7 +1444,7 @@ class SaagieApi:
         if not graph_pipeline.list_job_nodes:
             graph_pipeline.to_pipeline_graph_input()
         if cron_scheduling:
-            gql_scheduling_payload.append(f'isScheduled: true')
+            gql_scheduling_payload.append('isScheduled: true')
 
             if croniter.is_valid(cron_scheduling):
                 gql_scheduling_payload.append(f'cronScheduling: "{cron_scheduling}"')
@@ -1258,7 +1457,7 @@ class SaagieApi:
                 raise RuntimeError("Please specify a correct timezone")
 
         else:
-            gql_scheduling_payload.append(f'isScheduled: false')
+            gql_scheduling_payload.append('isScheduled: false')
 
         gql_scheduling_payload_str = ", ".join(gql_scheduling_payload)
 
@@ -1310,7 +1509,8 @@ class SaagieApi:
         if not graph_pipeline.list_job_nodes:
             graph_pipeline.to_pipeline_graph_input()
 
-        params = {'id': pipeline_id, 'jobNodes': graph_pipeline.list_job_nodes, 'conditionNodes': graph_pipeline.list_conditions_nodes, 'releaseNote': release_note}
+        params = {'id': pipeline_id, 'jobNodes': graph_pipeline.list_job_nodes,
+                  'conditionNodes': graph_pipeline.list_conditions_nodes, 'releaseNote': release_note}
 
         return self.client.execute(gql(gql_upgrade_pipeline), variable_values=params)
 
@@ -1324,18 +1524,13 @@ class SaagieApi:
             Name of your project
         Returns
         -------
-        dict
+        str
             Pipeline UUID
         """
-        projects = self.get_projects_info()["projects"]
-        project = list(filter(lambda p: p["name"] == project_name, projects))
-        if project:
-            project_id = project[0]["id"]
-            pipelines = self.get_project_pipelines(project_id, instances_limit=1)["project"]["pipelines"]
-            pipeline = list(filter(lambda j: j["name"] == pipeline_name, pipelines))
-            if pipeline:
-                return pipeline[0]["id"]
-            else:
-                raise NameError(f"pipeline {pipeline_name} does not exist.")
+        project_id = self.get_project_id(project_name)
+        pipelines = self.get_project_pipelines(project_id, instances_limit=1)["project"]["pipelines"]
+        pipeline = list(filter(lambda j: j["name"] == pipeline_name, pipelines))
+        if pipeline:
+            return pipeline[0]["id"]
         else:
-            raise NameError(f"Project {project_name} does not exist or you don't have permission to see it.")
+            raise NameError(f"pipeline {pipeline_name} does not exist.")
